@@ -1,6 +1,6 @@
 /* 
  * GDA common library
- * Copyright (C) 1998 - 2004 The GNOME Foundation.
+ * Copyright (C) 1998 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -889,27 +889,26 @@ gda_data_model_commit_update (GdaDataModel *model)
 }
 
 static gchar *
-export_to_separated (GdaDataModel *model, gchar sep)
+export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, gchar sep)
 {
 	GString *str;
 	gchar *retval;
-	gint cols, c, rows, r;
+	gint c, rows, r;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	str = g_string_new ("");
-	cols = gda_data_model_get_n_columns (model);
 	rows = gda_data_model_get_n_rows (model);
 
 	for (r = 0; r < rows; r++) {
 		if (r > 0)
 			str = g_string_append_c (str, '\n');
 
-		for (c = 0; c < cols; c++) {
+		for (c = 0; c < nb_cols; c++) {
 			GdaValue *value;
 			gchar *txt;
 
-			value = (GdaValue *) gda_data_model_get_value_at (model, c, r);
+			value = (GdaValue *) gda_data_model_get_value_at (model, cols[c], r);
 			if (gda_value_get_type (value) == GDA_VALUE_TYPE_BOOLEAN)
 				txt = g_strdup (gda_value_get_boolean (value) ? "TRUE" : "FALSE");
 			else
@@ -931,39 +930,47 @@ export_to_separated (GdaDataModel *model, gchar sep)
 }
 
 /**
- * gda_data_model_to_comma_separated
+ * gda_data_model_to_text_separated
  * @model: a #GdaDataModel object.
+ * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @nb_cols: the number of columns in @cols
+ * @sep: a char separator
  *
- * Converts the given model into a comma-separated series of rows.
+ * Converts the given model into a char-separated series of rows.
+ * Examples of @sep are ',' for comma separated export and '\t' for tab
+ * separated export
  *
  * Returns: the representation of the model. You should free this
  * string when you no longer need it.
  */
 gchar *
-gda_data_model_to_comma_separated (GdaDataModel *model)
+gda_data_model_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, gchar sep)
 {
-	return export_to_separated (model, ',');
+	if (cols)
+		return export_to_text_separated (model, cols, nb_cols, sep);
+	else {
+		gchar *retval;
+		gint *rcols, rnb_cols, i;
+
+		g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+		rnb_cols = gda_data_model_get_n_columns (model);
+		rcols = g_new (gint, rnb_cols);
+		for (i = 0; i < rnb_cols; i++)
+			rcols[i] = i;
+		retval = export_to_text_separated (model, rcols, rnb_cols, sep);
+		g_free (rcols);
+
+		return retval;
+	}
 }
 
-/**
- * gda_data_model_to_tab_separated
- * @model: a #GdaDataModel object.
- *
- * Converts the given model into a tab-separated series of rows.
- *
- * Returns: the representation of the model. You should free this
- * string when you no longer need it.
- */
-gchar *
-gda_data_model_to_tab_separated (GdaDataModel *model)
-{
-	return export_to_separated (model, '\t');
-}
 
 /**
  * gda_data_model_to_xml
  * @model: a #GdaDataModel object.
- * @standalone: whether ... 
+ * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @nb_cols: the number of columns in @cols
  *
  * Converts the given model into a XML representation.
  *
@@ -971,42 +978,25 @@ gda_data_model_to_tab_separated (GdaDataModel *model)
  * string when you no longer need it.
  */
 gchar *
-gda_data_model_to_xml (GdaDataModel *model, gboolean standalone)
+gda_data_model_to_xml (GdaDataModel *model, const gint *cols, gint nb_cols, const gchar *name)
 {
 	xmlChar *xml_contents;
 	xmlNodePtr xml_node;
 	gchar *xml;
 	gint size;
+	xmlDocPtr xml_doc;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	xml_node = gda_data_model_to_xml_node (model, "exported_model");
-
-	if (standalone) {
-		xmlDocPtr xml_doc;
-		xmlNodePtr xml_root, xml_tables;
+	xml_node = gda_data_model_to_xml_node (model, cols, nb_cols, name);
+	xml_doc = xmlNewDoc ("1.0");
+	xmlDocSetRootElement (xml_doc, xml_node);
 	
-		xml_doc = xmlNewDoc ("1.0");
-		
-		xml_root = xmlNewDocNode (xml_doc, NULL, "database", NULL);
-		xmlDocSetRootElement (xml_doc, xml_root);
-
-		xml_tables = xmlNewChild (xml_root, NULL, "tables", NULL);
-		xmlAddChild (xml_tables, xml_node);
-
-		xmlDocDumpMemory (xml_doc, &xml_contents, &size);
-		xmlFreeDoc (xml_doc);
-		
-		xml = g_strdup (xml_contents);
-		xmlFree (xml_contents);
-	}
-	else {
-		xmlBufferPtr xml_buf = xmlBufferCreate ();
-		xmlNodeDump (xml_buf, NULL, xml_node, 0, 0); 
-		xml_contents = (xmlChar *) xmlBufferContent (xml_buf);
-		xmlBufferFree (xml_buf);
-		xml = g_strdup (xml_contents);
-	}
+	xmlDocDumpMemory (xml_doc, &xml_contents, &size);
+	xmlFreeDoc (xml_doc);
+	
+	xml = g_strdup (xml_contents);
+	xmlFree (xml_contents);
 
 	return xml;
 }
@@ -1029,6 +1019,8 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
 /**
  * gda_data_model_to_xml_node
  * @model: a #GdaDataModel object.
+ * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @nb_cols: the number of columns in @cols
  * @name: name to use for the XML resulting table.
  *
  * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
@@ -1036,25 +1028,39 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
  * Returns: a xmlNodePtr representing the whole data model.
  */
 xmlNodePtr
-gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
+gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, const gchar *name)
 {
 	xmlNodePtr node;
-	gint cols, rows, i;
+	gint rows, i;
+	gint *rcols, rnb_cols;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	node = xmlNewNode (NULL, "table");
-	if (name != NULL)
+	node = xmlNewNode (NULL, "data-array");
+	if (name)
 		xmlSetProp (node, "name", name);
+	else
+		xmlSetProp (node, "name", _("Exported Data"));
+
+	/* compute columns if not provided */
+	if (!cols) {
+		rnb_cols = gda_data_model_get_n_columns (model);
+		rcols = g_new (gint, rnb_cols);
+		for (i = 0; i < rnb_cols; i++)
+			rcols [i] = i;
+	}
+	else {
+		rcols = cols;
+		rnb_cols = nb_cols;
+	}
 
 	/* set the table structure */
 	rows = gda_data_model_get_n_rows (model);
-	cols = gda_data_model_get_n_columns (model);
-	for (i = 0; i < cols; i++) {
+	for (i = 0; i < rnb_cols; i++) {
 		GdaColumn *column;
 		xmlNodePtr field;
 
-		column = gda_data_model_describe_column (model, i);
+		column = gda_data_model_describe_column (model, rcols [i]);
 		if (!column) {
 			xmlFreeNode (node);
 			return NULL;
@@ -1083,11 +1089,11 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 		for (r = 0; r < rows; r++) {
 			row = xmlNewChild (data, NULL, "row", NULL);
 			xml_set_int (row, "position", r);
-			for (c = 0; c < cols; c++) {
+			for (c = 0; c < rnb_cols; c++) {
 				GdaValue *value;
 				gchar *str;
 
-				value = (GdaValue *) gda_data_model_get_value_at (model, c, r);
+				value = (GdaValue *) gda_data_model_get_value_at (model, rcols [c], r);
 				if (gda_value_get_type (value) == GDA_VALUE_TYPE_BOOLEAN)
 					str = g_strdup (gda_value_get_boolean (value) ? "TRUE" : "FALSE");
 				else
@@ -1101,6 +1107,9 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 		}
 	}
 
+	if (!cols)
+		g_free (rcols);
+
 	return node;
 }
 
@@ -1232,8 +1241,8 @@ gda_data_model_get_command_text (GdaDataModel *model)
 gboolean
 gda_data_model_set_command_text (GdaDataModel *model, const gchar *txt)
 {
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	g_return_if_fail (txt != NULL);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (txt != NULL, FALSE);
 	
 	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command)
 		return (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command) (model, txt, -1);
@@ -1275,10 +1284,151 @@ gda_data_model_get_command_type (GdaDataModel *model)
 gboolean
 gda_data_model_set_command_type (GdaDataModel *model, GdaCommandType type)
 {
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command)
 		return (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command) (model, NULL, type);
 	else
 		return FALSE;
 }
 
+/**
+ * gda_data_model_dump
+ * @model: a #GdaDataModel.
+ * @to_stream: where to dump the data model
+ *
+ * Dumps a textual representation of the @model to the @to_stream stream
+ */
+void
+gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
+{
+	gchar *str;
+
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+	g_return_if_fail (to_stream);
+
+	str = gda_data_model_dump_as_string (model);
+	g_fprintf (to_stream, "%s", str);
+	g_free (str);
+}
+
+/**
+ * gda_data_model_dump_as_string
+ * @model: a #GdaDataModel.
+ *
+ * Dumps a textual representation of the @model into a new string
+ *
+ * Returns: a new string.
+ */
+gchar *
+gda_data_model_dump_as_string (GdaDataModel *model)
+{
+	GString *string;
+	gchar *offstr, *str;
+	gint n_cols, n_rows;
+	gint *cols_size;
+	gboolean *cols_is_num;
+	gchar *sep_col  = " | ";
+	gchar *sep_row  = "-+-";
+	gchar sep_fill = '-';
+	gint i, j;
+	const GdaValue *value;
+
+	gint offset = 0;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+        /* string for the offset */
+        offstr = g_new0 (gchar, offset+1);
+	memset (offstr, ' ', offset);
+
+	/* compute the columns widths: using column titles... */
+	n_cols = gda_data_model_get_n_columns (model);
+	n_rows = gda_data_model_get_n_rows (model);
+	cols_size = g_new0 (gint, n_cols);
+	cols_is_num = g_new0 (gboolean, n_cols);
+	
+	for (i = 0; i < n_cols; i++) {
+		GdaColumn *gdacol;
+		GdaValueType coltype;
+
+		str = gda_data_model_get_column_title (model, i);
+		cols_size [i] = g_utf8_strlen (str, -1);
+
+		gdacol = gda_data_model_describe_column (model, i);
+		coltype = gda_column_get_gdatype (gdacol);
+		if ((coltype == GDA_VALUE_TYPE_BIGINT) ||
+		    (coltype == GDA_VALUE_TYPE_BIGUINT) ||
+		    (coltype == GDA_VALUE_TYPE_INTEGER) ||
+		    (coltype == GDA_VALUE_TYPE_NUMERIC) ||
+		    (coltype == GDA_VALUE_TYPE_SINGLE) ||
+		    (coltype == GDA_VALUE_TYPE_SMALLINT) ||
+		    (coltype == GDA_VALUE_TYPE_SMALLUINT) ||
+		    (coltype == GDA_VALUE_TYPE_TINYINT) ||
+		    (coltype == GDA_VALUE_TYPE_TINYUINT) ||
+		    (coltype == GDA_VALUE_TYPE_UINTEGER))
+			cols_is_num [i] = TRUE;
+		else
+			cols_is_num [i] = FALSE;
+	}
+
+	/* ... and using column data */
+	for (j = 0; j < n_rows; j++) {
+		for (i = 0; i < n_cols; i++) {
+			value = gda_data_model_get_value_at (model, i, j);
+			str = value ? gda_value_stringify (value) : g_strdup ("_null_");
+			cols_size [i] = MAX (cols_size [i], g_utf8_strlen (str, -1));
+			g_free (str);
+		}
+	}
+	
+	/* actual dumping of the contents: column titles...*/
+	string = g_string_new ("");
+	for (i = 0; i < n_cols; i++) {
+		gint j;
+		str = gda_data_model_get_column_title (model, i);
+		if (i != 0)
+			g_string_append_printf (string, "%s", sep_col);
+
+		g_string_append_printf (string, "%s", str);
+		for (j = 0; j < (cols_size [i] - g_utf8_strlen (str, -1)); j++)
+			g_string_append_c (string, ' ');
+	}
+	g_string_append_c (string, '\n');
+		
+	/* ... separation line ... */
+	for (i = 0; i < n_cols; i++) {
+		if (i != 0)
+			g_string_append_printf (string, "%s", sep_row);
+		for (j = 0; j < cols_size [i]; j++)
+			g_string_append_c (string, sep_fill);
+	}
+	g_string_append_c (string, '\n');
+
+	/* ... and data */
+	for (j = 0; j < n_rows; j++) {
+		for (i = 0; i < n_cols; i++) {
+			value = gda_data_model_get_value_at (model, i, j);
+			str = value ? gda_value_stringify (value) : g_strdup ("_null_");
+			if (i != 0)
+				g_string_append_printf (string, "%s", sep_col);
+			if (cols_is_num [i])
+				g_string_append_printf (string, "%*s", cols_size [i], str);
+			else {
+				gint j;
+				g_string_append_printf (string, "%s", str);
+				for (j = 0; j < (cols_size [i] - g_utf8_strlen (str, -1)); j++)
+					g_string_append_c (string, ' ');
+			}
+			g_free (str);
+		}
+		g_string_append_c (string, '\n');
+	}
+	g_free (cols_size);
+	g_free (cols_is_num);
+
+	g_free (offstr);
+	
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+}
