@@ -20,6 +20,7 @@
  */
 
 #include <libgda/gda-intl.h>
+#include <libgda/gda-config.h>
 #include "gda-xml-provider.h"
 
 const gchar *plugin_get_name (void);
@@ -43,8 +44,11 @@ GList *
 plugin_get_connection_params (void)
 {
 	GList *list = NULL;
-
-	list = g_list_append (list, g_strdup ("URI"));
+	
+	list = g_list_append (list,
+			      gda_provider_parameter_info_new_full ("URI", _("File Name"),
+								    _("Full path of the file containing the XML database"),
+								    GDA_VALUE_TYPE_STRING));
 
 	return list;
 }
